# Module to prepare the input parameters for GLOVE
# Kenji Sugase		2/10/10

use strict;
require Exporter;			# Use the Exporter package

package GlovePars;
my @ISA = qw(Exporter Autoloader);

sub mod_name {
	my $self  = shift;
	my $model = shift;		# Short model name
	my $name  = shift;		# Long model name
#	print STDERR "MODEL: Model \"$model\" added.\n";
	if (defined($name)) {
		$self->{NAME}->{uc($model)} = uc($name);
	} else {
		$self->{NAME}->{$self->{MODEL}};
	}
}

# Set parameter index and range
sub add_mod_par {
	my $self  = shift;
	my $model = uc(shift);		# Model name
	my $par   = uc(shift);		# Parameter name
	my $min   = shift;		# Minimum range
	my $max   = shift;		# Maximum range
	my $step  = shift;		# Grid size
	my $idx   = shift;		# Index for temperature, B0 and concentration
	my $range = "$min $max $step";
#	print STDERR "MODEL: Model \"$model\" parameter \"$par\" added.\n";
	$self->{RANGE}->{$par} = $range;
	if (defined($idx)) {
		$self->{PAR}->{$model}->{$par}->{TDEP} = substr($idx, 0, 1);	# Index for temperature
		$self->{PAR}->{$model}->{$par}->{FDEP} = substr($idx, 1, 1);	# Index for B0
		if (length($idx) == 3) {
			$self->{PAR}->{$model}->{$par}->{CDEP} = substr($idx, 2, 1);	# Index for concentation
		}
	} else {
		$self->{PAR}->{$model}->{$par}->{UNUSED};
	}
}

sub pick_model {
	my $self  = shift;
	my $model = uc(shift);
	if (exists($self->{PAR}->{$model})) {
		$self->{MODEL} = $model;
	} else {
		print STDERR "ERROR: Model \"$model\" not recognized.\n";
		print STDERR "Valid models:";
		foreach my $valid_model (keys(%{$self->{PAR}})) {
			print STDERR " $valid_model";
		}
		print STDERR "\n";
		exit(1);
	}
}

# Check whether the parameter name exists
sub exist_par {
	my $self  = shift;
	my $par   = uc(shift);		# Parameter name
	my $model = $self->{MODEL};
	return (exists($self->{PAR}->{$model}->{$par}));
}

sub change_min {
	my $self  = shift;
	my $par   = uc(shift);		# Parameter name
	my $min   = shift;		# Minimum range
	my @range = split(/\s+/, $self->{RANGE}->{$par});
	my $max   = $range[1];
	my $size  = $range[2];
	$self->{RANGE}->{$par} = "$min $max $size";
}

sub change_max {
	my $self  = shift;
	my $par   = uc(shift);		# Parameter name
	my $max   = shift;		# Maximum range
	my @range = split(/\s+/, $self->{RANGE}->{$par});
	my $min   = $range[0];
	my $size  = $range[2];
	$self->{RANGE}->{$par} = "$min $max $size";
}

sub change_size {
	my $self  = shift;
	my $par   = uc(shift);		# Parameter name
	my $size  = shift;		# Grid size
	my @range = split(/\s+/, $self->{RANGE}->{$par});
	my $min   = $range[0];
	my $max   = $range[1];
	$self->{RANGE}->{$par} = "$min $max $size";
}

sub remove_step {
	my $self  = shift;
	my $model = $self->{MODEL};
	foreach my $par (keys(%{$self->{PAR}->{$model}})) {
		my @range = split(/\s+/, $self->{RANGE}->{$par});
		my $min   = $range[0];
		my $max   = $range[1];
		$self->{RANGE}->{$par} = "$min $max";
	}
}

sub define_local {
	my $self     = shift;
	my $gi       = shift;		# Reference to Glove::Input
	my $gpar_flg = shift;		# Reference to hash of globals
	my $temp_idx = shift;		# Temperature index
	my $B0_idx   = shift;		# B0 Field index
	my $conc_idx = shift;		# Concentration index
	my $model    = $self->{MODEL};
	# Mark up the parameter names
	foreach my $par (keys(%{$self->{PAR}->{$model}})) {
		my $par_idx = $par;
		if ($self->{PAR}->{$model}->{$par}->{TDEP}) {
			$par_idx .= "_" . $temp_idx;
		}
		if ($self->{PAR}->{$model}->{$par}->{FDEP}) {
			$par_idx .= "_" . $B0_idx;
		}
		if ($self->{PAR}->{$model}->{$par}->{CDEP}) {
			if (!defined($conc_idx)) {
				die "ERROR: Model $model requires a concentration index\n";
			}
			$par_idx .= "_" . $conc_idx;
		}
		if (exists($gpar_flg->{$par})) { 
			$self->{GLOBAL}->{$par_idx} = $par; 
			$gi->add_par($par_idx, $par_idx); 
		} else { 
			$gi->add_par($par_idx, $self->{RANGE}->{$par}); 
		}
	}
}

sub define_local_RDD {
	my $self     = shift;
	my $gi       = shift;		# Reference to Glove::Input
	my $gpar_flg = shift;		# Reference to hash of globals
	my $B0_idx   = shift;		# B0 Field index
	my $conc_idx = shift;		# Concentration index
	my $model    = $self->{MODEL};
	# Mark up the parameter names
	foreach my $par (keys(%{$self->{PAR}->{$model}})) {
		my $par_idx = $par;
		if ($self->{PAR}->{$model}->{$par}->{FDEP}) {
			$par_idx .= "_" . $B0_idx;
		}
		if ($self->{PAR}->{$model}->{$par}->{CDEP}) {
			if (!defined($conc_idx)) {
				die "ERROR: Model $model requires a concentration index\n";
			}
			$par_idx .= "_" . $conc_idx;
		}
		if (exists($gpar_flg->{$par})) { 
			$self->{GLOBAL}->{$par_idx} = $par; 
			$gi->add_par($par_idx, $par_idx); 
		} else { 
			$gi->add_par($par_idx, $self->{RANGE}->{$par}); 
		}
	}
}

sub change_range {
	my $self     = shift;
	my $gi       = shift;		# Reference to Glove::Input
	my $par      = uc(shift);	# Parameter name
	my $range    = shift;		# New parameter range
	my $temp_idx = shift;		# Temperature index
	my $B0_idx   = shift;		# B0 Field index
	my $conc_idx = shift;		# Concentration index
	my $model    = $self->{MODEL};
	# Mark up the parameter names
	my $par_idx = $par;
	if ($self->{PAR}->{$model}->{$par}->{TDEP}) {
		$par_idx .= "_" . $temp_idx;
	}
	if ($self->{PAR}->{$model}->{$par}->{FDEP}) {
		$par_idx.= "_" . $B0_idx;
	}
	if ($self->{PAR}->{$model}->{$par}->{CDEP}) {
		if (!defined($conc_idx)) {
			die "ERROR: Model $model requires a concentration index\n";
		}
		$par_idx .= "_" . $conc_idx;
	}
	$gi->add_par($par_idx, $range); 
}

sub define_global {
	my $self = shift;
	my $gi   = shift;			# Reference to Glove::Input
	if (!defined($gi)) {
		die "ERROR: Bad call to GlovePars::define_global()\n";
	}
	if (!exists($self->{GLOBAL})) { return; }
	# Retrieve all global parameters
	foreach my $k (keys(%{$self->{GLOBAL}})) {
		my $par = $self->{GLOBAL}->{$k};
		if (!defined($self->{RANGE}->{$par})) {
			die "ERROR (GlovePar::define_global: Range of \"$par\" not defined.\n";
		}
		$gi->add_gpar($k, $self->{RANGE}->{$par});
	}
}
