# Module to parse the input file for GLOVE
# Jonathan C. Lansing	2/25/2005
# Kenji Sugase		9/3/2012

use strict;
require Exporter;		# Use the Exporter package

package Glove::Input;
my @ISA = qw(Exporter Autoloader);

sub write {
	use lib "$ENV{GLOVEDIR}/bin/perl_lib";
	use Sort::Naturally;

	my $self = shift(@_);
	my $file = shift(@_);
	my $line;
	if (!defined($file) || $file eq "STDOUT") {
		*OUT = *STDOUT;
	} else { 
		open(OUT, ">" . $file) || die "ERROR: Unable to create file \'$file\'\n";
	}

	# Write model
	print OUT "#################     Fitting Model      #################\n";
	print OUT "FUNCTION $self->{FUNC}\n";

	# Write methods
	print OUT "\n";
	print OUT "#################     Fitting Method     #################\n";
	print OUT "# METHOD GRID                                            #\n";
	print OUT "# METHOD ONE                                             #\n";
	print OUT "# METHOD ONEEX                                           #\n";
	print OUT "# METHOD RANDOM iteration                                #\n";
	print OUT "# METHOD MCMIN iteration scale                           #\n";
	print OUT "##########################################################\n";
		
	foreach $line (@{$self->{METHOD}}) {
		print OUT "$line\n";
	}

	# Write Monte-Carlo or Jacknife simulation
	print OUT "\n";
	print OUT "#################    Error Estimation    #################\n";
	print OUT "# JACK (no parameter)                                    #\n";
	print OUT "# MONTE iteration                                        #\n";
	print OUT "##########################################################\n";
	foreach $line (@{$self->{SIM}}) {
		print OUT "$line\n";
	}

	# Write instructions of optional calculations
	print OUT "\n";
	print OUT "#################  Optional Calculation  #################\n";
	foreach $line (@{$self->{CALC}}) {
		print OUT "CALC $line\n";
	}

	# Write plot instructions
	print OUT "\n";
	print OUT "#################    Plot Instruction    #################\n";
	foreach $line (@{$self->{PLOT}}) {
		print OUT "PLOT $line\n";
	}

	# Write XMGR plot options
	print OUT "\n";
	print OUT "#################      XMGR Option       #################\n";
	foreach $line (@{$self->{XMGR}}) {
		print OUT "$line\n";
	}

	# Write global parameters
	print OUT "\n";
	print OUT "#################    Global Parameter    #################\n";
	print OUT "# GPAR   keyword start (end) (grid size) (initial)       #\n";
	print OUT "# GREST  keyword start  end  (grid size) (initial)       #\n";
#	print OUT "# GFIXED keyword start                                   #\n";
	print OUT "##########################################################\n";
	foreach $line (sort keys(%{$self->{PAR}->{GLOBAL}})) {
		print OUT "GPAR $line $self->{PAR}->{GLOBAL}->{$line}\n";
	}

	# Write data sets
	print OUT "\n";
	print OUT "#==================================================================\n";
	print OUT "#                           Data Set\n";
	print OUT "#==================================================================\n";
	print OUT "#################    local Parameter     #################\n";
	print OUT "# PAR   parameter start (end) (grid size) (initial)      #\n";
	print OUT "# REST  parameter start  end  (grid size) (initial)      #\n";
	print OUT "# FIXED parameter start                                  #\n";
	print OUT "##########################################################\n";
	print OUT "\n";
	foreach my $label (nsort keys(%{$self->{DATA}})) {
		print OUT "SET $label\n";
		foreach my $par_idx (nsort keys(%{$self->{PAR}->{$label}})) {
			if (defined($self->{FIXED}->{$label}->{$par_idx})) {
				print OUT "FIXED $par_idx $self->{PAR}->{$label}->{$par_idx}\n";
			} else {
				print OUT "PAR $par_idx $self->{PAR}->{$label}->{$par_idx}\n";
			}
		}
		foreach $line (@{$self->{DATA}->{$label}}) {
			print OUT "$line\n";
		}
	}
	if (defined($file) && $file ne "STDOUT") {
		close(OUT);
	}
}

sub set_function {
	my $self  = shift(@_);
	my $model = shift(@_);
	if (defined($model)) {
		$self->{FUNC} = $model;
	}
}

sub add_header {
	my $self   = shift(@_);
	my $header = shift(@_);
	if (!defined($self->{HEADER})) {
		$self->{HEADER} = [];
	}
	if (defined($header)) {
		push(@{$self->{HEADER}}, $header);
	}
}

sub add_method {
	my $self   = shift(@_);
	my $method = shift(@_);
	if (!defined($self->{METHOD})) {
		$self->{METHOD} = [];
	}
	if (defined($method)) {
		push(@{$self->{METHOD}}, $method);
	}
}

sub set_sim {
	my $self = shift(@_);
	my $sim  = shift(@_);
	if (!defined($self->{SIM})) {
		$self->{SIM} = [];
	}
	if (defined($sim)) {
		push(@{$self->{SIM}}, $sim);
	}
}

sub add_calc {
	my $self = shift(@_);
	my $calc = shift(@_);
	if (!defined($self->{CALC})) {
		$self->{CALC} = [];
	}
	if (defined($calc)) {
		push(@{$self->{CALC}}, $calc);
	}
}

sub add_plot {
	my $self = shift(@_);
	my $plot = shift(@_);
	if (!defined($self->{PLOT})) {
		$self->{PLOT} = [];
	}
	if (defined($plot)) {
		push(@{$self->{PLOT}}, $plot);
	}
}

sub add_xmgr {
	my $self = shift(@_);
	my $xmgr = shift(@_);
	if (!defined($self->{XMGR})) {
		$self->{XMGR} = [];
	}
	if (defined($xmgr)) {
		push(@{$self->{XMGR}}, $xmgr);
	}
}

sub new_set {
	my $self  = shift(@_);
	my $label = shift(@_);
	if (!defined($label)) {
		return;
	}
	$self->{CURR} = $label;	
}

sub add_gpar {
	my $self     = shift(@_);
	my $par_name = shift(@_);
	my $range    = shift(@_);
	if (!defined($par_name) || !defined($range)) {
		return;
	}
	$self->{PAR}->{GLOBAL}->{$par_name} = $range;
}

sub add_par {
	my $self     = shift(@_);
	my $par_name = shift(@_);
	my $range    = shift(@_);
	my $label    = $self->{CURR};
	if (!defined($par_name) || !defined($range) || !defined($label)) {
		return;
	}
	$self->{PAR}->{$self->{CURR}}->{$par_name} = $range;
	delete($self->{FIXED}->{$label}->{$par_name});
}

sub add_fixed {
	my $self     = shift(@_);
	my $par_name = shift(@_);
	my $range    = shift(@_);
	my $label    = $self->{CURR};
	if (!defined($par_name) || !defined($range) || !defined($label)) {
		return;
	}
	$self->{FIXED}->{$label}->{$par_name} = 1;
	$self->{PAR}->{$label}->{$par_name} = $range;
}

sub add_data {
	my $self     = shift(@_);
	my $par_name = shift(@_);
	if (!defined($par_name) || !defined($self->{CURR})) {
		return;
	}
	push(@{$self->{DATA}->{$self->{CURR}}}, $par_name);
}
